@@ -98,15 +98,15 @@ pyMainItf::log(cmLogKind kind, const char* format, ...)
 
 // From cmCnx thread
 void
-pyMainItf::notifyNewRemoteBuffer(bsVec<u8>& buffer)
+pyMainItf::notifyNewRemoteBuffer(int streamId, bsVec<u8>& buffer)
 {
-    _live->storeNewRemoteBuffer(buffer);
+    _live->storeNewRemoteBuffer(streamId, buffer);
 }
 
 
 // From cmCnx thread via live control
 void
-pyMainItf::notifyNewFrozenThreadState(u64 frozenThreadBitmap)
+pyMainItf::notifyNewFrozenThreadState(int streamId, u64 frozenThreadBitmap)
 {
     notifyScript(); // Update the script side first, to ensure all is synchronised with freeze points
     _ntf->notifyNewFrozenThreadState(frozenThreadBitmap);
@@ -115,7 +115,7 @@ pyMainItf::notifyNewFrozenThreadState(u64 frozenThreadBitmap)
 
 // From cmCnx thread via live control
 void
-pyMainItf::notifyCommandAnswer(plPriv::plRemoteStatus status, const bsString& answer)
+pyMainItf::notifyCommandAnswer(int streamId, plPriv::plRemoteStatus status, const bsString& answer)
 {
     _ntf->notifyCommandAnswer(status, answer.toChar());
 }
@@ -123,7 +123,7 @@ pyMainItf::notifyCommandAnswer(plPriv::plRemoteStatus status, const bsString& an
 
 // From cmCnx thread via live control
 void
-pyMainItf::notifyNewCli(u32 nameIdx, int paramSpecIdx, int descriptionIdx)
+pyMainItf::notifyNewCli(int streamId, u32 nameIdx, int paramSpecIdx, int descriptionIdx)
 {
     _batchedClis.push_back({_recording->getString(nameIdx).toChar(), _recording->getString(paramSpecIdx).toChar(),
             _recording->getString(descriptionIdx).toChar()});
@@ -131,30 +131,27 @@ pyMainItf::notifyNewCli(u32 nameIdx, int paramSpecIdx, int descriptionIdx)
 
 
 bool
-pyMainItf::notifyRecordStarted(const bsString& appName, const bsString& buildName, s64 timeTickOrigin, double tickToNs,
-                               const cmTlvs& options)
+pyMainItf::notifyRecordStarted(const cmStreamInfo& infos, s64 timeTickOrigin, double tickToNs)
 {
     bsString errorMsg;
-    _recording->beginRecord(appName, buildName, timeTickOrigin, tickToNs, options,
-                            RECORD_CACHE_MB, errorMsg, false);
+    _recording->beginRecord(infos.appName, infos, timeTickOrigin, tickToNs,
+                            false, RECORD_CACHE_MB, _recordFilename, false, errorMsg);
     if(!errorMsg.empty()) {
         notifyErrorForDisplay(ERROR_GENERIC, errorMsg);
         return false;
     }
 
     // Compute the spec hashes, now that we know the string hash size
-    _isStringHashShort  = options.values[PL_TLV_HAS_SHORT_STRING_HASH];
-    _areStringsExternal = options.values[PL_TLV_HAS_EXTERNAL_STRING];
+    _isStringHashShort  = infos.tlvs[PL_TLV_HAS_SHORT_STRING_HASH];
+    _areStringsExternal = infos.tlvs[PL_TLV_HAS_EXTERNAL_STRING];
 
-    _ntf->notifyRecordStarted(appName.toChar(), buildName.toChar(), _areStringsExternal, _isStringHashShort, !options.values[PL_TLV_HAS_NO_CONTROL]);
+    _ntf->notifyRecordStarted(infos.appName.toChar(), infos.buildName.toChar(), _areStringsExternal, _isStringHashShort, !infos.tlvs[PL_TLV_HAS_NO_CONTROL]);
 
     std::lock_guard<std::mutex> lk(_mx);
     // Clean the event specifications, so they can manage the new record
+    // Changing the state of the external strings or using another auto instrumented program requires new specifications (computed hash will change)
     for(Spec& f : _specs) {
-        f.threadHash = 0;
-        for(SpecElemToken& set : f.parentPath) set.hash = 0;
         for(SpecElem& se : f.elems) {
-            for(SpecElemToken& set : se.tokens) set.hash = 0;
             se.resolution = NO_ELEMENTS_SEEN;
         }
         f.parentElemIdx = -1;
@@ -171,17 +168,17 @@ pyMainItf::notifyRecordStarted(const bsString& appName, const bsString& buildNam
 
 
 bool
-pyMainItf::notifyNewEvents(plPriv::EventExt* events, int eventQty)
+pyMainItf::notifyNewEvents(int streamId, plPriv::EventExt* events, int eventQty, s64 shortDateSyncTick)
 {
-    return _recording->storeNewEvents(events, eventQty);
+    return _recording->storeNewEvents(streamId, events, eventQty, shortDateSyncTick);
 }
 
 
 void
-pyMainItf::notifyNewString(const bsString& newString, u64 hash)
+pyMainItf::notifyNewString(int streamId, const bsString& newString, u64 hash)
 {
     // Record the new string
-    const bsString& storedString = _recording->storeNewString(newString, hash);
+    const bsString& storedString = _recording->storeNewString(streamId, newString, hash);
 
     // Batch it for external notification
     _batchedStrings.push_back({ hash, storedString.toChar() });
@@ -189,13 +186,20 @@ pyMainItf::notifyNewString(const bsString& newString, u64 hash)
 
 
 void
-pyMainItf::notifyNewCollectionTick(void)
+pyMainItf::notifyNewCollectionTick(int streamId)
 {
     _didCollectionTickOccurred = true;
     notifyScript();
 }
 
 
+void
+pyMainItf::notifyNewStream(const cmStreamInfo& infos)
+{
+    _recording->notifyNewStream(infos);
+}
+
+
 void
 pyMainItf::notifyNewThread(int threadId, u64 nameHash)
 {
@@ -393,7 +397,7 @@ pyMainItf::clearBufferedEvents(void)
 
 
 void
-pyMainItf::addSpec(const char* threadName, pyiSpec* parentPath, pyiSpec* elemArray, int elemQty)
+pyMainItf::addSpec(const char* threadName, u64 threadHash, pyiSpec* parentPath, pyiSpec* elemArray, int elemQty)
 {
     std::lock_guard<std::mutex> lk(_mx);
 
@@ -404,24 +408,25 @@ pyMainItf::addSpec(const char* threadName, pyiSpec* parentPath, pyiSpec* elemArr
 
     // Copy the fields
     f.threadName = threadName;
+    f.threadHash = threadHash;
     f.parentPath.resize(parentPath->pathQty);
     for(int i=0; i<parentPath->pathQty; ++i) {
-        const char* s = parentPath->path[i];
-        if     (!strcmp(s, "."))  f.parentPath[i] = { "", 3 };
-        else if(!strcmp(s, "**")) f.parentPath[i] = { "", 2 };
-        else if(!strcmp(s, "*"))  f.parentPath[i] = { "", 1 };
-        else                      f.parentPath[i] = {  s, 0 };
+        const pyiPath& s = parentPath->path[i];
+        if     (!strcmp(s.name, "."))  f.parentPath[i] = { "", 3 };
+        else if(!strcmp(s.name, "**")) f.parentPath[i] = { "", 2 };
+        else if(!strcmp(s.name, "*"))  f.parentPath[i] = { "", 1 };
+        else                           f.parentPath[i] = { s.name, 0, s.hash };
     }
     f.elems.resize(elemQty);
     for(int j=0; j<elemQty; ++j) {
         f.elems[j].resolution = NO_ELEMENTS_SEEN;
         f.elems[j].tokens.resize(elemArray[j].pathQty);
         for(int i=0; i<elemArray[j].pathQty; ++i) {
-            const char* s = elemArray[j].path[i];
-            if     (!strcmp(s, "."))  f.elems[j].tokens[i] = { "", 3 };
-            else if(!strcmp(s, "**")) f.elems[j].tokens[i] = { "", 2 };
-            else if(!strcmp(s, "*"))  f.elems[j].tokens[i] = { "", 1 };
-            else                      f.elems[j].tokens[i] = {  s, 0 };
+            const pyiPath& s = elemArray[j].path[i];
+            if     (!strcmp(s.name, "."))  f.elems[j].tokens[i] = { "", 3 };
+            else if(!strcmp(s.name, "**")) f.elems[j].tokens[i] = { "", 2 };
+            else if(!strcmp(s.name, "*"))  f.elems[j].tokens[i] = { "", 1 };
+            else                           f.elems[j].tokens[i] = { s.name, 0, s.hash };
        }
     }
 
@@ -445,16 +450,25 @@ void
 pyMainItf::computeSpecHashes(Spec& f)
 {
     // Thread name
-    if(f.threadName.empty()) f.threadHash = 0;
-    else f.threadHash = _isStringHashShort? bsHash32String(f.threadName.toChar()) : bsHashString(f.threadName.toChar());
+    if(f.threadName.empty()) {
+        f.threadHash = 0;
+    }
+    else if(f.threadHash==0) {
+        f.threadHash = _isStringHashShort? bsHash32String(f.threadName.toChar()) : bsHashString(f.threadName.toChar());
+    }
+
     // Parent path
     for(SpecElemToken& set : f.parentPath) {
-        set.hash = _isStringHashShort? bsHash32String(set.name.toChar()) : bsHashString(set.name.toChar());
+        if(set.hash==0) {
+            set.hash = _isStringHashShort? bsHash32String(set.name.toChar()) : bsHashString(set.name.toChar());
+        }
     }
     // Events path
     for(SpecElem& se : f.elems) {
         for(SpecElemToken& set : se.tokens) {
-            set.hash = _isStringHashShort? bsHash32String(set.name.toChar()) : bsHashString(set.name.toChar());
+            if(set.hash==0) {
+                set.hash = _isStringHashShort? bsHash32String(set.name.toChar()) : bsHashString(set.name.toChar());
+            }
         }
     }
 }
@@ -616,9 +630,13 @@ pyMainItf::notifyFilteredEvent(int elemIdx, int flags, u64 nameHash, s64 dateNs,
 
 
 void
-pyMainItf::setRecordingConfig(bool isEnabled, const char* forcedFilename)
+pyMainItf::setRecordFilename(const char* recordFilename)
 {
-    _recording->setRecordingConfig(isEnabled, forcedFilename);
+    if(recordFilename) {
+        _recordFilename = recordFilename;
+    } else {
+        _recordFilename.clear();
+    }
 }
 
 
